__author__  = 'BiblioAnalysis team'
__license__ = 'MIT'


from ScopusApyJson.GLOBALS import *
from ScopusApyJson.json_utils import *
from ScopusApyJson.json_parser import *
from ScopusApyJson.api_manager import *
from ScopusApyJson.main import *